#include "tetris-struct.h"

#include <stdlib.h>

Grid gridInit(){
    Grid g;
    g.grid = calloc(g.width, sizeof(Square*));

    for(int i = 0; i < g.width; i++){
        g.grid[i] = calloc(g.height, sizeof(Square));
    }
    
    return g;
}

Piece pieceInit(){
    Piece p;
    p.squares = calloc(p.size, sizeof(Square));

    return p;
}

bool gameOver(Grid grid){
    return height(grid) == grid.height;
}

int height(Grid g){
    int height = 0;
    int height_p;

    for(int i = 0; i < g.width; i++){
        height_p = heightColumn(g.grid[i], g.height);

        if(height_p > height){
            height = height_p;
        }
    }
}

int heightColumn(Square* column, int size){
    int height = 0;
    int y;

    for(int i = 0; i < size; i++){
        if(column[i] != NULL){
            y = column[i]->position[1]; 
        
            if(y+1 > height){
                height = y+1;
            }
        }
        
    }

    return height;
}

bool canMove(Piece piece, Grid grid){
    int x, y;

    for(int i = 0; i < piece.size; i++){
        if(!canMoveSquare(piece.squares[i], grid)){
            return false;
        }
    }
    return true;
}

bool canMoveSquare(Square sq, Grid g){
    int x = sq->position[0];
    int y = sq->position[1];
    
    return (y != 0 || y == heightColumn(g.grid[x], g.height));
}

void move(Piece piece){
    for(int i = 0; i < piece.size; i++){
        moveSquare(piece.squares[i]);
    }
}

void moveSquare(Square sq){
    sq->position[1] --;
}

void addPieceToGrid(Piece piece, Grid grid){
    for(int i = 0; i < piece.size; i++){
        addSquareToGrid(piece.squares[i], grid);
    }
}

void addSquareToGrid(Square sq, Grid g){
    int x = sq->position[0];
    int y = sq->position[1];
    
    g.grid[x][y] = sq;
}